<?php
/**
 * Plugin Name: wp-mat
 * Plugin URI: https://git.autistici.org/noblogs/wp-mat
 * Description: Process uploaded files through MAT
 * Version: 0.1.0
 * Author: Autistici/Inventati
 * Author URI: https://www.autistici.org/
 * License: MIT
 * License URI: http://opensource.org/licenses/MIT
 */

if (!defined("WP_MAT_ENDPOINT")) {
    define("WP_MAT_ENDPOINT", "http://localhost:7111/api/cleanup");
}

// Check if the mime type is one of those we want to clean up.
// Use a blacklist, currently empty. Send everything by default
// except video.
function wp_mat_is_supported_mime_type($mimetype) {
    if (substr($mimetype, 0, 6) === "video/") {
        return false;
    }
    return true;
}

// Make a POST request to the MAT service with the uploaded file and
// store the results into '$output_path'.
function wp_mat_issue_request($path, $mimetype, $output_path) {
    $upfile = new CURLFile($path, $mimetype, 'input');
    
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_URL, WP_MAT_ENDPOINT);
    curl_setopt($curl, CURLOPT_USERAGENT, 'Wordpress');
    curl_setopt($curl, CURLOPT_POST, true);
    curl_setopt($curl, CURLOPT_POSTFIELDS, array(
        "file" => $upfile,
        "mime_type" => $mimetype,
    ));

    if (($fp = fopen($output_path, "wb")) === false) {
        throw new Exception("fopen error for filename $output_path");
    }
    curl_setopt($curl, CURLOPT_FILE, $fp);
    curl_setopt($curl, CURLOPT_BINARYTRANSFER, true);
    
    $ret = curl_exec($curl);

    fclose($fp);
    curl_close($curl);

    return $ret;
}

function wp_mat_handle_upload_prefilter($file) {
    if (isset($file['error']) && $file['error']) {
        return $file;
    }

    // Autodetect MIME type if not passed by the browser.
    if (!$file['type']) {
        $file['type'] = mime_content_type($file['tmp_name']);
    }

    // Check if it's one of the MIME types we support.
    if (!wp_mat_is_supported_mime_type($file['type'])) {
        error_log("wp-mat: unsupported mime type ".$file['type']);
        return $file;
    }

    // Issue a request to the MAT API, and if successful replace the
    // file with the sanitized one.
    $new_tmp_file = tempnam("", "wp_mat_");
    if (wp_mat_issue_request($file['tmp_name'], $file['type'], $new_tmp_file)) {
        // Rename the cleaned up file to the original file, or WP
        // will not pass the is_uploaded_file() check in wp_handle_upload().
        unlink($file['tmp_name']);
        rename($new_tmp_file, $file['tmp_name']);

        $new_file = array(
            "name" => $file['name'],
            "type" => $file['type'],
            "tmp_name" => $file['tmp_name'],
            "size" => filesize($file['tmp_name']),
            "error" => 0,
        );
        error_log("wp-mat: successfully cleaned file=".json_encode($new_file));
        $file = $new_file;
    } else {
        error_log("wp-mat: server error for file=".json_encode($file));
        @unlink($new_tmp_file);
    }

    return $file;
}

add_filter('wp_handle_upload_prefilter', 'wp_mat_handle_upload_prefilter');

// This is generally useful for debugging.
if (!function_exists('wp_handle_upload_error')) {
    function wp_handle_upload_error(&$file, $message) {
        error_log("wp_handle_upload_error: ".$message);
        return array( "error" => $message );
    }
}
